local addonName = "StrataFix"
StrataFix = {}
local addon = StrataFix 

local frame = CreateFrame("Button", addonName.."HiddenFrame", UIParent)
local revision = tonumber(("$Revision: 5 $"):match("%d+"))

local function chatMsg(msg)
     DEFAULT_CHAT_FRAME:AddMessage(addonName..": "..msg)
end

-- frames that should never be messed with,
-- specifically those containing child frames with a deliberately lower strata
addon.blacklist = {
  -- ["ExampleFrameName"] = true,
}
for i=1,10 do
  addon.blacklist["ChatFrame"..i.."EditBox"] = true
end

function addon:FixHelper(lvl, ...)
  local n = select("#",...)
  local changes = 0
  for i=1,n do
    local frame = select(i, ...)
    changes = changes + addon:Fix(frame, lvl)
  end
  return changes
end

function addon:Fix(frame,lvl)
  if not frame:IsShown() or addon.blacklist[frame:GetName() or ""] then
    return 0
  end
  local curlvl = frame:GetFrameLevel()
  local changes = 0
  lvl = lvl or curlvl
  if curlvl < lvl then
     curlvl = lvl
     frame:SetFrameLevel(curlvl)
     changes = changes + 1
  end
  changes = changes + addon:FixHelper(curlvl+1, frame:GetChildren())
  return changes
end

function addon:FixAll(silent)
  local changes = 0
  local frame = EnumerateFrames(); -- Get the first frame
  while frame do
    if ( frame:IsToplevel() and frame:IsVisible() ) then
      changes = changes + addon:Fix(frame)
    end
    frame = EnumerateFrames(frame); -- Get the next frame
  end

  if not silent and changes ~= addon.lastchanges then
    chatMsg("Frame strata fixed! ("..changes.." changes)")
  end
  addon.lastchanges = changes
end

hooksecurefunc("ShowUIPanel",function() addon:FixAll(not addon.debug) end)

local upd = 0
function StrataFix_OnUpdate(frame, elapsed)
  if InCombatLockdown() then return end
  upd = upd + elapsed
  if upd < 0.5 then 
    return
  else
    upd = 0
  end
  addon:FixAll(not addon.debug)
end

function StrataFix_OnEvent(frame, event, name, ...)
   if event == "ADDON_LOADED" and name == addonName then
       chatMsg("Revision "..revision.." loaded.")
   elseif event == "PLAYER_REGEN_ENABLED" then 
       -- This event is called when the player exits combat
       frame:SetScript("OnUpdate", StrataFix_OnUpdate);
   elseif event == "PLAYER_REGEN_DISABLED" then
       -- This event is called when we enter combat
       frame:SetScript("OnUpdate", nil);
   end
end


frame:SetScript("OnEvent", StrataFix_OnEvent);
if not InCombatLockdown() then
  frame:SetScript("OnUpdate", StrataFix_OnUpdate);
end
frame:RegisterEvent("ADDON_LOADED")
frame:RegisterEvent("PLAYER_REGEN_ENABLED")
frame:RegisterEvent("PLAYER_REGEN_DISABLED")



